package com.abernal.prizy



import static org.springframework.http.HttpStatus.*
import grails.transaction.Transactional

@Transactional(readOnly = true)
class ProductController {

    static allowedMethods = [save: "POST", update: "PUT", delete: "DELETE"]
    def productService
    /*def scaffold = Product*/

    def redirectTest() {
        render "/product/index"
    }

    def index(Integer max) {
        params.max = Math.min(max ?: 10, 100)
        respond Product.list(params), model:[productInstanceCount: Product.count()]
    }

    def show(Product productInstance) {

        BigDecimal averagePrice = productService.getAveragePrice(productInstance.id)
        BigDecimal lowestPrice = productService.getLowestPrice(productInstance.id)
        BigDecimal highestPrice = productService.getHighestPrice(productInstance.id)
        BigDecimal idealPrice   = productService.getIdealPrice(productInstance.id)
        Integer numberOfPricesCollected  = productService.getNumberOfPricesCollected(productInstance.id)

        respond productInstance, model:[averagePrice: averagePrice, lowestPrice: lowestPrice, highestPrice: highestPrice, idealPrice: idealPrice, numberOfPricesCollected: numberOfPricesCollected]
    }

    def create() {
        respond new Product(params)
    }

    @Transactional
    def save(Product productInstance) {
        if (productInstance == null) {
            notFound()
            return
        }

        if (productInstance.hasErrors()) {
            respond productInstance.errors, view:'create'
            return
        }

        productInstance.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.created.message', args: [message(code: 'product.label', default: 'Product'), productInstance.id])
                redirect productInstance
            }
            '*' { respond productInstance, [status: CREATED] }
        }
    }

    def edit(Product productInstance) {
        respond productInstance
    }

    @Transactional
    def update(Product productInstance) {
        if (productInstance == null) {
            notFound()
            return
        }

        if (productInstance.hasErrors()) {
            respond productInstance.errors, view:'edit'
            return
        }

        productInstance.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.updated.message', args: [message(code: 'Product.label', default: 'Product'), productInstance.id])
                redirect productInstance
            }
            '*'{ respond productInstance, [status: OK] }
        }
    }

    @Transactional
    def delete(Product productInstance) {

        if (productInstance == null) {
            notFound()
            return
        }

        productInstance.delete flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.deleted.message', args: [message(code: 'Product.label', default: 'Product'), productInstance.id])
                redirect action:"index", method:"GET"
            }
            '*'{ render status: NO_CONTENT }
        }
    }

    def searchResults = {

        List<Product> productInstanceList

        if(params.containsKey('barcode') && params.barcode != ""){

            def c = Product.createCriteria()
            def results = c {
                sqlRestriction "cast( barcode AS char( 256 ) ) like '%${params?.barcode}%'"
            }

            productInstanceList = results;
        }else{
             productInstanceList = Product.list()
        }


        render(view:'index', model:['productInstanceList':productInstanceList])
    }

    protected void notFound() {
        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.not.found.message', args: [message(code: 'product.label', default: 'Product'), params.id])
                redirect action: "index", method: "GET"
            }
            '*'{ render status: NOT_FOUND }
        }
    }
}
